import { Subscribable, ObservableArray, Observable } from "knockout";
import { CssProperty } from "./types";
import { EventListeners } from "./event";
import { Attributes, AttributesSubscribable } from "./attributes";
import { Style, StyleSubscribable } from "./style";
export interface ViewModelOptions {
    attributes?: Attributes | AttributesSubscribable;
    classNames?: Array<string> | Subscribable<Array<string>>;
    style?: Style | StyleSubscribable;
    visible?: boolean | Subscribable<boolean>;
    disabled?: boolean | Subscribable<boolean>;
    innerHtml?: string | Subscribable<string>;
    innerText?: string | Subscribable<string>;
    addViewModelToDataObject?: boolean;
    userSelectable?: boolean;
    dispose?: () => void;
    refresh?: () => void;
    eventListeners?: EventListeners;
    viewType?: string;
    focus?: () => void;
}
export interface IObservableProperty<TValue, TSetter = TValue> extends Subscribable<TValue> {
    (): TValue;
    set: (value: TSetter) => void;
}
export interface IObservableArrayProperty<TItem, TSetter> extends ObservableArray<TItem> {
    set: (items: Array<TSetter>) => void;
}
export declare class ViewModel implements ViewModelOptions {
    constructor(options?: ViewModelOptions);
    readonly classNames: string[] | Subscribable<Array<string>>;
    readonly attributes: Attributes | AttributesSubscribable;
    readonly style: Style | StyleSubscribable;
    readonly visible: boolean | Subscribable<boolean>;
    readonly disabled: boolean | Subscribable<boolean>;
    readonly innerHtml: string | Subscribable<string>;
    readonly innerText: string | Subscribable<string>;
    readonly addViewModelToDataObject: boolean;
    readonly userSelectable: boolean;
    readonly dispose: () => void;
    readonly refresh: () => void;
    readonly eventListeners: EventListeners;
    readonly viewType: string;
    protected getStyle(...cssProperties: Array<CssProperty>): Style;
    protected getStyleValue(cssProperty: CssProperty): any;
    setClasses(classNames: string[], replace: boolean): void;
    setAttributes(attributes: Attributes): void;
    setStyle(style: Style): void;
    setVisible(visible: boolean): void;
    setDisabled(disabled: boolean): void;
    setInnerHtml(innerHtml: string): void;
    setInnerText(innerText: string): void;
    removeClasses(classNames: string[]): void;
    focus(): void;
    handleWebSocketMessage: (message: any) => void;
    protected setObservableProperty(property: any, value: any): void;
    protected createObservableProperty<TValue, TSetter = TValue>(value?: TValue | Observable<TValue>, setter?: (value: TSetter) => void, notifyAlways?: boolean): IObservableProperty<TValue, TSetter>;
    protected createObservableArrayProperty<TItem, TSetter = TItem>(items?: Array<TItem> | ObservableArray<TItem>, setter?: (items: Array<TSetter>) => void): IObservableArrayProperty<TItem, TSetter>;
}
export declare function extendViewModel<TResult = ViewModelOptions, TTarget = ViewModelOptions, TSource = ViewModelOptions>(target: TTarget, source: TSource): TResult;
export declare function applyViewModelToElement(viewModel: ViewModelOptions, element: HTMLElement): void;
